@@ -1,8 +1,7 @@
-use std::iter;
-
-use crate::dictionary::Dictionary;
-
-use super::{Composition, ConversionEngine, Interval};
+use crate::{
+    conversion::{Composition, ConversionEngine, Interval, Outcome},
+    dictionary::{Dictionary, LookupStrategy},
+};
 
 /// Simple engine does not perform any intelligent conversion.
 #[derive(Debug, Default)]
@@ -12,11 +11,7 @@ impl SimpleEngine {
     pub fn new() -> SimpleEngine {
         SimpleEngine
     }
-    pub fn convert<'a>(
-        &'a self,
-        dict: &'a dyn Dictionary,
-        comp: &'a Composition,
-    ) -> impl Iterator<Item = Vec<Interval>> + Clone + 'a {
+    pub fn convert<'a>(&'a self, dict: &'a dyn Dictionary, comp: &'a Composition) -> Vec<Outcome> {
         let mut intervals = vec![];
 
         for (i, sym) in comp.symbols().iter().enumerate() {
@@ -32,13 +27,13 @@ impl SimpleEngine {
                     start: i,
                     end: i + 1,
                     is_phrase: false,
-                    str: sym.to_char().unwrap().to_string().into_boxed_str(),
+                    text: sym.to_char().unwrap().to_string().into_boxed_str(),
                 });
             } else {
-                let phrase = dict.lookup_first_phrase(
-                    &[sym.to_syllable().unwrap()],
-                    crate::dictionary::LookupStrategy::Standard,
-                );
+                let phrase = dict
+                    .lookup(&[sym.to_syllable().unwrap()], LookupStrategy::Standard)
+                    .first()
+                    .cloned();
                 let phrase_str = phrase.map_or_else(
                     || sym.to_syllable().unwrap().to_string(),
                     |phrase| phrase.to_string(),
@@ -47,37 +42,35 @@ impl SimpleEngine {
                     start: i,
                     end: i + 1,
                     is_phrase: true,
-                    str: phrase_str.into_boxed_str(),
+                    text: phrase_str.into_boxed_str(),
                 })
             }
         }
         intervals.extend_from_slice(comp.selections());
         intervals.sort_by_key(|int| int.start);
-        iter::once(intervals)
+        vec![Outcome {
+            intervals,
+            log_prob: 0.0,
+        }]
     }
 }
 
 impl ConversionEngine for SimpleEngine {
-    fn convert<'a>(
-        &'a self,
-        dict: &'a dyn Dictionary,
-        comp: &'a Composition,
-    ) -> Box<dyn Iterator<Item = Vec<Interval>> + 'a> {
-        Box::new(SimpleEngine::convert(self, dict, comp))
+    fn convert<'a>(&'a self, dict: &'a dyn Dictionary, comp: &'a Composition) -> Vec<Outcome> {
+        SimpleEngine::convert(self, dict, comp)
     }
 }
 
 #[cfg(test)]
 mod tests {
+    use super::SimpleEngine;
     use crate::{
-        conversion::{Composition, Interval, Symbol},
+        conversion::{Composition, Interval, Outcome, Symbol},
         dictionary::{Dictionary, TrieBuf},
         syl,
         zhuyin::Bopomofo::*,
     };
 
-    use super::SimpleEngine;
-
     fn test_dictionary() -> impl Dictionary {
         TrieBuf::from([
             (vec![syl![G, U, O, TONE2]], vec![("國", 1)]),
@@ -130,8 +123,11 @@ mod tests {
         let engine = SimpleEngine::new();
         let composition = Composition::new();
         assert_eq!(
-            Some(Vec::<Interval>::new()),
-            engine.convert(&dict, &composition).next()
+            vec![Outcome {
+                intervals: vec![],
+                log_prob: 0.0
+            }],
+            engine.convert(&dict, &composition)
         );
     }
 
@@ -139,8 +135,7 @@ mod tests {
     #[test]
     fn convert_zero_length_entry() {
         let mut dict = test_dictionary();
-        let dict_mut = dict.as_dict_mut().unwrap();
-        dict_mut.add_phrase(&[], ("", 0).into()).unwrap();
+        dict.add_phrase(&[], ("", 0).into()).unwrap();
         let engine = SimpleEngine::new();
         let mut composition = Composition::new();
         for sym in [
@@ -150,21 +145,24 @@ mod tests {
             composition.push(sym);
         }
         assert_eq!(
-            Some(vec![
-                Interval {
-                    start: 0,
-                    end: 1,
-                    is_phrase: true,
-                    str: "測".into()
-                },
-                Interval {
-                    start: 1,
-                    end: 2,
-                    is_phrase: true,
-                    str: "試".into()
-                },
-            ]),
-            engine.convert(&dict, &composition).next()
+            vec![Outcome {
+                intervals: vec![
+                    Interval {
+                        start: 0,
+                        end: 1,
+                        is_phrase: true,
+                        text: "測".into()
+                    },
+                    Interval {
+                        start: 1,
+                        end: 2,
+                        is_phrase: true,
+                        text: "試".into()
+                    },
+                ],
+                log_prob: 0.0
+            }],
+            engine.convert(&dict, &composition)
         );
     }
 
@@ -184,45 +182,48 @@ mod tests {
             composition.push(sym);
         }
         assert_eq!(
-            Some(vec![
-                Interval {
-                    start: 0,
-                    end: 1,
-                    is_phrase: true,
-                    str: "國".into()
-                },
-                Interval {
-                    start: 1,
-                    end: 2,
-                    is_phrase: true,
-                    str: "民".into()
-                },
-                Interval {
-                    start: 2,
-                    end: 3,
-                    is_phrase: true,
-                    str: "大".into()
-                },
-                Interval {
-                    start: 3,
-                    end: 4,
-                    is_phrase: true,
-                    str: "會".into()
-                },
-                Interval {
-                    start: 4,
-                    end: 5,
-                    is_phrase: true,
-                    str: "代".into()
-                },
-                Interval {
-                    start: 5,
-                    end: 6,
-                    is_phrase: true,
-                    str: "表".into()
-                },
-            ]),
-            engine.convert(&dict, &composition).next()
+            vec![Outcome {
+                intervals: vec![
+                    Interval {
+                        start: 0,
+                        end: 1,
+                        is_phrase: true,
+                        text: "國".into()
+                    },
+                    Interval {
+                        start: 1,
+                        end: 2,
+                        is_phrase: true,
+                        text: "民".into()
+                    },
+                    Interval {
+                        start: 2,
+                        end: 3,
+                        is_phrase: true,
+                        text: "大".into()
+                    },
+                    Interval {
+                        start: 3,
+                        end: 4,
+                        is_phrase: true,
+                        text: "會".into()
+                    },
+                    Interval {
+                        start: 4,
+                        end: 5,
+                        is_phrase: true,
+                        text: "代".into()
+                    },
+                    Interval {
+                        start: 5,
+                        end: 6,
+                        is_phrase: true,
+                        text: "表".into()
+                    },
+                ],
+                log_prob: 0.0
+            }],
+            engine.convert(&dict, &composition)
         );
     }
 }
