@@ -5,12 +5,8 @@ use std::{
 
 use log::error;
 
-use crate::zhuyin::SyllableSlice;
-
-use super::{
-    Dictionary, DictionaryInfo, DictionaryMut, Entries, LookupStrategy, Phrase,
-    UpdateDictionaryError,
-};
+use super::{Dictionary, DictionaryInfo, Entries, LookupStrategy, Phrase, UpdateDictionaryError};
+use crate::zhuyin::Syllable;
 
 /// A collection of dictionaries that returns the union of the lookup results.
 /// # Examples
@@ -18,7 +14,11 @@ use super::{
 /// ```
 /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
 ///
-/// use chewing::{dictionary::{Layered, TrieBuf, Dictionary, LookupStrategy, Phrase}, syl, zhuyin::Bopomofo};
+/// use chewing::{
+///     dictionary::{Dictionary, Layered, LookupStrategy, Phrase, TrieBuf},
+///     syl,
+///     zhuyin::Bopomofo,
+/// };
 ///
 /// let sys_dict = TrieBuf::from([(
 ///     vec![syl![Bopomofo::C, Bopomofo::E, Bopomofo::TONE4]],
@@ -33,13 +33,13 @@ use super::{
 /// assert_eq!(
 ///     [
 ///         ("側", 1, 0).into(),
-///         ("冊", 100, 0).into(),
+///         ("冊", 101, 0).into(),
 ///         ("測", 1, 0).into(),
 ///         ("策", 100, 0).into(),
 ///     ]
 ///     .into_iter()
 ///     .collect::<Vec<Phrase>>(),
-///     dict.lookup_all_phrases(&[
+///     dict.lookup(&[
 ///         syl![Bopomofo::C, Bopomofo::E, Bopomofo::TONE4]
 ///     ], LookupStrategy::Standard),
 /// );
@@ -68,7 +68,11 @@ impl Layered {
 impl Dictionary for Layered {
     /// Lookup phrases from all underlying dictionaries.
     ///
-    /// Phrases are ordered by their first apperance in the underlying dictionaries.
+    /// Phrases are ordered by their first apperance in the underlying
+    /// dictionaries.
+    ///
+    /// When a phrase appears in multiple dictionaries, the final
+    /// frequency is the sum of all frequency in all dictionaries.
     ///
     /// Pseudo code
     ///
@@ -77,16 +81,11 @@ impl Dictionary for Layered {
     /// Foreach d in d_layers
     ///   Foreach phrase, freq in d.lookup_syllables()
     ///     If phrase in phrases
-    ///       Set phrases[phrase].freq = freq
+    ///       Set phrases[phrase].freq += freq
     ///     Else
     ///       Add phrases <- (phrase, freq)
     /// ```
-    fn lookup_first_n_phrases(
-        &self,
-        syllables: &dyn SyllableSlice,
-        first: usize,
-        strategy: LookupStrategy,
-    ) -> Vec<Phrase> {
+    fn lookup(&self, syllables: &[Syllable], strategy: LookupStrategy) -> Vec<Phrase> {
         let mut sort_map: BTreeMap<String, usize> = BTreeMap::new();
         let mut phrases: Vec<Phrase> = Vec::new();
 
@@ -94,12 +93,19 @@ impl Dictionary for Layered {
             .iter()
             .chain(iter::once(&self.user_dict))
             .for_each(|d| {
-                for phrase in d.lookup_all_phrases(syllables, strategy) {
+                for phrase in d.lookup(syllables, strategy) {
                     debug_assert!(!phrase.as_str().is_empty());
                     match sort_map.entry(phrase.to_string()) {
                         Entry::Occupied(entry) => {
                             let index = *entry.get();
-                            phrases[index] = phrase.clone();
+                            phrases[index].freq += phrase.freq;
+                            phrases[index].last_used =
+                                match (phrases[index].last_used, phrase.last_used) {
+                                    (Some(orig), Some(new)) => Some(u64::max(orig, new)),
+                                    (Some(orig), None) => Some(orig),
+                                    (None, Some(new)) => Some(new),
+                                    (None, None) => None,
+                                };
                         }
                         Entry::Vacant(entry) => {
                             entry.insert(phrases.len());
@@ -108,7 +114,6 @@ impl Dictionary for Layered {
                     }
                 }
             });
-        phrases.truncate(first);
         phrases
     }
 
@@ -135,47 +140,29 @@ impl Dictionary for Layered {
         None
     }
 
-    fn as_dict_mut(&mut self) -> Option<&mut dyn DictionaryMut> {
-        self.user_dict.as_dict_mut()
-    }
-}
-
-impl DictionaryMut for Layered {
     fn reopen(&mut self) -> Result<(), UpdateDictionaryError> {
-        if let Some(writer) = self.user_dict.as_dict_mut() {
-            writer.reopen()
-        } else {
-            Ok(())
-        }
+        self.user_dict.reopen()
     }
 
     fn flush(&mut self) -> Result<(), UpdateDictionaryError> {
-        if let Some(writer) = self.user_dict.as_dict_mut() {
-            writer.flush()
-        } else {
-            Ok(())
-        }
+        self.user_dict.flush()
     }
 
     fn add_phrase(
         &mut self,
-        syllables: &dyn SyllableSlice,
+        syllables: &[Syllable],
         phrase: Phrase,
     ) -> Result<(), UpdateDictionaryError> {
         if phrase.as_str().is_empty() {
             error!("BUG! added phrase is empty");
             return Ok(());
         }
-        if let Some(writer) = self.user_dict.as_dict_mut() {
-            writer.add_phrase(syllables, phrase)
-        } else {
-            Ok(())
-        }
+        self.user_dict.add_phrase(syllables, phrase)
     }
 
     fn update_phrase(
         &mut self,
-        syllables: &dyn SyllableSlice,
+        syllables: &[Syllable],
         phrase: Phrase,
         user_freq: u32,
         time: u64,
@@ -184,23 +171,16 @@ impl DictionaryMut for Layered {
             error!("BUG! added phrase is empty");
             return Ok(());
         }
-        if let Some(writer) = self.user_dict.as_dict_mut() {
-            writer.update_phrase(syllables, phrase, user_freq, time)
-        } else {
-            Ok(())
-        }
+        self.user_dict
+            .update_phrase(syllables, phrase, user_freq, time)
     }
 
     fn remove_phrase(
         &mut self,
-        syllables: &dyn SyllableSlice,
+        syllables: &[Syllable],
         phrase_str: &str,
     ) -> Result<(), UpdateDictionaryError> {
-        if let Some(writer) = self.user_dict.as_dict_mut() {
-            writer.remove_phrase(syllables, phrase_str)
-        } else {
-            Ok(())
-        }
+        self.user_dict.remove_phrase(syllables, phrase_str)
     }
 }
 
@@ -211,17 +191,15 @@ mod tests {
         io::{Cursor, Seek},
     };
 
+    use super::Layered;
     use crate::{
         dictionary::{
-            Dictionary, DictionaryBuilder, DictionaryMut, LookupStrategy, Phrase, Trie, TrieBuf,
-            TrieBuilder,
+            Dictionary, DictionaryBuilder, LookupStrategy, Phrase, Trie, TrieBuf, TrieBuilder,
         },
         syl,
         zhuyin::Bopomofo,
     };
 
-    use super::Layered;
-
     #[test]
     fn test_entries() -> Result<(), Box<dyn Error>> {
         let sys_dict = TrieBuf::from([(
@@ -278,21 +256,23 @@ mod tests {
         let dict = Layered::new(vec![Box::new(sys_dict)], Box::new(user_dict));
         assert_eq!(
             Some(("側", 1, 0).into()),
-            dict.lookup_first_phrase(
+            dict.lookup(
                 &vec![syl![Bopomofo::C, Bopomofo::E, Bopomofo::TONE4]],
                 LookupStrategy::Standard
-            ),
+            )
+            .first()
+            .cloned(),
         );
         assert_eq!(
             [
                 ("側", 1, 0).into(),
-                ("冊", 100, 0).into(),
+                ("冊", 101, 0).into(),
                 ("測", 1, 0).into(),
                 ("策", 100, 0).into(),
             ]
             .into_iter()
             .collect::<Vec<Phrase>>(),
-            dict.lookup_all_phrases(
+            dict.lookup(
                 &vec![syl![Bopomofo::C, Bopomofo::E, Bopomofo::TONE4]],
                 LookupStrategy::Standard
             ),
@@ -323,35 +303,36 @@ mod tests {
         let mut dict = Layered::new(vec![Box::new(sys_dict)], Box::new(user_dict));
         assert_eq!(
             Some(("側", 1, 0).into()),
-            dict.lookup_first_phrase(
+            dict.lookup(
                 &vec![syl![Bopomofo::C, Bopomofo::E, Bopomofo::TONE4]],
                 LookupStrategy::Standard
-            ),
+            )
+            .first()
+            .cloned(),
         );
         assert_eq!(
             [
                 ("側", 1, 0).into(),
-                ("冊", 100, 0).into(),
+                ("冊", 101, 0).into(),
                 ("測", 1, 0).into(),
                 ("策", 100, 0).into(),
             ]
             .into_iter()
             .collect::<Vec<Phrase>>(),
-            dict.lookup_all_phrases(
+            dict.lookup(
                 &vec![syl![Bopomofo::C, Bopomofo::E, Bopomofo::TONE4]],
                 LookupStrategy::Standard
             ),
         );
         let _ = dict.about();
-        assert!(dict.as_dict_mut().is_none());
-        assert!(dict.reopen().is_ok());
-        assert!(dict.flush().is_ok());
+        assert!(dict.reopen().is_err());
+        assert!(dict.flush().is_err());
         assert!(
             dict.add_phrase(
                 &[syl![Bopomofo::C, Bopomofo::E, Bopomofo::TONE4]],
                 ("冊", 100).into()
             )
-            .is_ok()
+            .is_err()
         );
         assert!(
             dict.update_phrase(
@@ -360,11 +341,11 @@ mod tests {
                 0,
                 0,
             )
-            .is_ok()
+            .is_err()
         );
         assert!(
             dict.remove_phrase(&[syl![Bopomofo::C, Bopomofo::E, Bopomofo::TONE4]], "冊")
-                .is_ok()
+                .is_err()
         );
         Ok(())
     }
