@@ -1,27 +1,29 @@
 //! Pinyin
 
+use super::{KeyBehavior, SyllableEditor};
 use crate::{
-    editor::keyboard::{KeyCode, KeyEvent},
+    input::{KeyboardEvent, keysym},
     zhuyin::{Bopomofo, Syllable},
 };
 
-use super::{KeyBehavior, SyllableEditor};
-
 const MAX_PINYIN_LEN: usize = 10;
 
 /// TODO: docs
 #[derive(Debug, Default, Clone, Copy, PartialEq, Eq)]
 pub enum PinyinVariant {
-    /// TODO: docs
+    /// [Hanyu Pinyin](https://en.wikipedia.org/wiki/Pinyin), or simply pinyin
     #[default]
     HanyuPinyin,
-    /// TODO: docs
+    /// [THL, Taíwan Huáyǔ Luómǎ Pinyin](https://pinyin.thl.tw/)
     ThlPinyin,
-    /// TODO: docs
+    /// [MPS2, Mandarin Phonetic Symbols II](https://pinyin.info/romanization/mps2)
     Mps2Pinyin,
 }
 
-/// TODO: docs
+/// Pinyin
+///
+/// Current support types: [`PinyinVariant`](enum@PinyinVariant)
+///
 #[derive(Default, Debug, Clone)]
 pub struct Pinyin {
     key_seq: String,
@@ -70,37 +72,38 @@ impl Pinyin {
 }
 
 impl SyllableEditor for Pinyin {
-    fn key_press(&mut self, key: KeyEvent) -> KeyBehavior {
-        if self.key_seq.is_empty() && !key.code.is_atoz() {
+    fn key_press(&mut self, key: KeyboardEvent) -> KeyBehavior {
+        let ksym = &key.ksym;
+        if self.key_seq.is_empty() && !ksym.is_atoz() {
             return KeyBehavior::KeyError;
         }
         if ![
-            KeyCode::Space,
-            KeyCode::N1,
-            KeyCode::N2,
-            KeyCode::N3,
-            KeyCode::N4,
-            KeyCode::N5,
+            keysym::SYM_SPACE,
+            keysym::SYM_1,
+            keysym::SYM_2,
+            keysym::SYM_3,
+            keysym::SYM_4,
+            keysym::SYM_5,
         ]
-        .contains(&key.code)
+        .contains(ksym)
         {
             if self.key_seq.len() == MAX_PINYIN_LEN {
                 // buffer is full, ignore this keystroke
                 return KeyBehavior::NoWord;
             }
-            if !key.unicode.is_ascii_alphabetic() {
+            if !ksym.is_ascii() {
                 return KeyBehavior::KeyError;
             }
-            self.key_seq.push(key.unicode);
+            self.key_seq.push(ksym.to_unicode());
             return KeyBehavior::Absorb;
         }
 
-        let tone = match key.code {
+        let tone = match ksym.to_digit() {
             // KeyCode::Space | KeyCode::N1 => Some(Bopomofo::TONE1),
-            KeyCode::N2 => Some(Bopomofo::TONE2),
-            KeyCode::N3 => Some(Bopomofo::TONE3),
-            KeyCode::N4 => Some(Bopomofo::TONE4),
-            KeyCode::N5 => Some(Bopomofo::TONE5),
+            Some(2) => Some(Bopomofo::TONE2),
+            Some(3) => Some(Bopomofo::TONE3),
+            Some(4) => Some(Bopomofo::TONE4),
+            Some(5) => Some(Bopomofo::TONE5),
             _ => None,
         };
 
@@ -160,24 +163,20 @@ impl SyllableEditor for Pinyin {
         /* Hanyu empty rime
          * ㄓ/ㄔ/ㄕ/ㄖ/ㄗ/ㄘ/ㄙ + -i, -i is empty rime, not ㄧ
          * */
-        match self.variant {
-            PinyinVariant::HanyuPinyin => {
-                if matches!(
-                    (medial, rime),
-                    (Some(Bopomofo::I), None) | (None, Some(Bopomofo::I))
-                ) {
-                    match initial {
-                        Some(Bopomofo::ZH) | Some(Bopomofo::CH) | Some(Bopomofo::SH)
-                        | Some(Bopomofo::R) | Some(Bopomofo::Z) | Some(Bopomofo::C)
-                        | Some(Bopomofo::S) => {
-                            medial.take();
-                            rime.take();
-                        }
-                        _ => (),
-                    }
+        if self.variant == PinyinVariant::HanyuPinyin
+            && matches!(
+                (medial, rime),
+                (Some(Bopomofo::I), None) | (None, Some(Bopomofo::I))
+            )
+        {
+            match initial {
+                Some(Bopomofo::ZH) | Some(Bopomofo::CH) | Some(Bopomofo::SH)
+                | Some(Bopomofo::R) | Some(Bopomofo::Z) | Some(Bopomofo::C) | Some(Bopomofo::S) => {
+                    medial.take();
+                    rime.take();
                 }
+                _ => (),
             }
-            _ => {}
         }
 
         /* Hanyu uan/un/u :
@@ -185,8 +184,8 @@ impl SyllableEditor for Pinyin {
          * ㄐ/ㄑ/ㄒ + -un,  -un is ㄩㄣ, not ㄨㄣ
          * ㄐ/ㄑ/ㄒ + -u,   -u is ㄧ, not ㄨ
          */
-        match self.variant {
-            PinyinVariant::HanyuPinyin => match initial {
+        if self.variant == PinyinVariant::HanyuPinyin {
+            match initial {
                 Some(Bopomofo::J) | Some(Bopomofo::Q) | Some(Bopomofo::X) => {
                     match (medial, rime) {
                         (Some(Bopomofo::U), Some(Bopomofo::AN))
@@ -198,8 +197,7 @@ impl SyllableEditor for Pinyin {
                     };
                 }
                 _ => (),
-            },
-            _ => {}
+            }
         }
 
         /* THL/MPS2 s/sh/c/ch/j :
@@ -270,7 +268,7 @@ impl SyllableEditor for Pinyin {
         KeyBehavior::Commit
     }
 
-    fn fuzzy_key_press(&mut self, key: KeyEvent) -> KeyBehavior {
+    fn fuzzy_key_press(&mut self, key: KeyboardEvent) -> KeyBehavior {
         self.key_press(key)
     }
 
@@ -349,9 +347,8 @@ macro_rules! fin {
 
 mod table {
 
-    use crate::{syl, zhuyin::Bopomofo::*};
-
     use super::{AmbiguousMapEntry, FinalMapEntry, InitialMapEntry};
+    use crate::{syl, zhuyin::Bopomofo::*};
 
     pub(super) const COMMON_MAPPING: [AmbiguousMapEntry; 18] = [
         // Special cases for WG
@@ -530,67 +527,68 @@ mod table {
 
 #[cfg(test)]
 mod tests {
+    use super::Pinyin;
     use crate::{
-        editor::{
-            keyboard::{AnyKeyboardLayout, KeyCode, KeyboardLayout},
-            zhuyin_layout::SyllableEditor,
+        editor::zhuyin_layout::SyllableEditor,
+        input::{
+            KeyboardEvent,
+            keysym::{self, Keysym},
         },
         syl,
         zhuyin::Bopomofo,
     };
 
-    use super::Pinyin;
+    fn map_key(ksym: Keysym) -> KeyboardEvent {
+        KeyboardEvent::builder().ksym(ksym).build()
+    }
 
     #[test]
     fn hanyu_empty_rime_zi() {
-        let keyboard = AnyKeyboardLayout::qwerty();
         let mut hanyu = Pinyin::hanyu();
 
-        hanyu.key_press(keyboard.map(KeyCode::Z));
-        hanyu.key_press(keyboard.map(KeyCode::I));
-        hanyu.key_press(keyboard.map(KeyCode::N1));
+        hanyu.key_press(map_key(keysym::SYM_LOWER_Z));
+        hanyu.key_press(map_key(keysym::SYM_LOWER_I));
+        hanyu.key_press(map_key(keysym::SYM_1));
 
         assert_eq!(syl![Bopomofo::Z], hanyu.read());
     }
 
     #[test]
     fn hanyu_empty_rime_zhi() {
-        let keyboard = AnyKeyboardLayout::qwerty();
         let mut hanyu = Pinyin::hanyu();
 
-        hanyu.key_press(keyboard.map(KeyCode::Z));
-        hanyu.key_press(keyboard.map(KeyCode::H));
-        hanyu.key_press(keyboard.map(KeyCode::I));
-        hanyu.key_press(keyboard.map(KeyCode::N1));
+        hanyu.key_press(map_key(keysym::SYM_LOWER_Z));
+        hanyu.key_press(map_key(keysym::SYM_LOWER_H));
+        hanyu.key_press(map_key(keysym::SYM_LOWER_I));
+        hanyu.key_press(map_key(keysym::SYM_1));
 
         assert_eq!(syl![Bopomofo::ZH], hanyu.read());
     }
 
     #[test]
     fn hanyu_uan_un_u() {
-        let keyboard = AnyKeyboardLayout::qwerty();
         let mut hanyu = Pinyin::hanyu();
 
-        hanyu.key_press(keyboard.map(KeyCode::J));
-        hanyu.key_press(keyboard.map(KeyCode::U));
-        hanyu.key_press(keyboard.map(KeyCode::A));
-        hanyu.key_press(keyboard.map(KeyCode::N));
-        hanyu.key_press(keyboard.map(KeyCode::N1));
+        hanyu.key_press(map_key(Keysym::from('j')));
+        hanyu.key_press(map_key(Keysym::from('u')));
+        hanyu.key_press(map_key(Keysym::from('a')));
+        hanyu.key_press(map_key(Keysym::from('n')));
+        hanyu.key_press(map_key(Keysym::from('1')));
 
         assert_eq!(syl![Bopomofo::J, Bopomofo::IU, Bopomofo::AN], hanyu.read());
 
         hanyu.clear();
-        hanyu.key_press(keyboard.map(KeyCode::Q));
-        hanyu.key_press(keyboard.map(KeyCode::U));
-        hanyu.key_press(keyboard.map(KeyCode::N));
-        hanyu.key_press(keyboard.map(KeyCode::N1));
+        hanyu.key_press(map_key(Keysym::from('q')));
+        hanyu.key_press(map_key(Keysym::from('u')));
+        hanyu.key_press(map_key(Keysym::from('n')));
+        hanyu.key_press(map_key(Keysym::from('1')));
 
         assert_eq!(syl![Bopomofo::Q, Bopomofo::IU, Bopomofo::EN], hanyu.read());
 
         hanyu.clear();
-        hanyu.key_press(keyboard.map(KeyCode::X));
-        hanyu.key_press(keyboard.map(KeyCode::U));
-        hanyu.key_press(keyboard.map(KeyCode::N1));
+        hanyu.key_press(map_key(Keysym::from('x')));
+        hanyu.key_press(map_key(Keysym::from('u')));
+        hanyu.key_press(map_key(Keysym::from('1')));
 
         assert_eq!(syl![Bopomofo::X, Bopomofo::IU], hanyu.read());
     }
