@@ -330,21 +330,25 @@ void test_del_bopomofo_as_mode_switch()
     ctx = chewing_new();
     start_testcase(ctx);
 
-    type_keystroke_by_string(ctx, "2k");        /* ㄉㄜ */
+    type_keystroke_by_string(ctx, "2k72k");        /* ㄉㄜ˙ ㄉㄜ */
     ok_bopomofo_buffer(ctx, "\xe3\x84\x89\xe3\x84\x9c" /* ㄉㄜ */ );
-    ok_preedit_buffer(ctx, "");
+    ok_preedit_buffer(ctx, "的");
     chewing_set_ChiEngMode(ctx, SYMBOL_MODE);
     ok_bopomofo_buffer(ctx, "");
-    ok_preedit_buffer(ctx, "");
+    ok_preedit_buffer(ctx, "的");
+
+    // can enter English after mode switch
+    type_keystroke_by_string(ctx, "test");
+    ok_preedit_buffer(ctx, "的test");
 
     chewing_set_ChiEngMode(ctx, CHINESE_MODE);
 
     type_keystroke_by_string(ctx, "ji");        /* ㄨㄛ */
     ok_bopomofo_buffer(ctx, "\xe3\x84\xa8\xe3\x84\x9b" /* ㄨㄛ */ );
-    ok_preedit_buffer(ctx, "");
+    ok_preedit_buffer(ctx, "的test");
     chewing_set_ChiEngMode(ctx, SYMBOL_MODE);
     ok_bopomofo_buffer(ctx, "");
-    ok_preedit_buffer(ctx, "");
+    ok_preedit_buffer(ctx, "的test");
 
     chewing_delete(ctx);
 }
@@ -474,6 +478,97 @@ void test_select_candidate_second_page_rewind()
     chewing_delete(ctx);
 }
 
+void test_select_candidate_shift_cursor()
+{
+    ChewingContext *ctx;
+
+    clean_userphrase();
+
+    ctx = chewing_new();
+    start_testcase(ctx);
+    chewing_set_maxChiSymbolLen(ctx, 16);
+    chewing_set_autoShiftCur(ctx, 1);
+
+    type_keystroke_by_string(ctx, "fu g;418 vu84s84xu3");
+    ok_preedit_buffer(ctx, "七上八下那裡");
+
+    type_keystroke_by_string(ctx, "<H>");
+
+    type_keystroke_by_string(ctx, "<D>1");
+    ok_preedit_buffer(ctx, "七上八下那裡");
+
+    type_keystroke_by_string(ctx, "<D><D>2");
+
+    ok_preedit_buffer(ctx, "七上八下納裡");
+
+    type_keystroke_by_string(ctx, "<D>2");
+
+    ok_preedit_buffer(ctx, "七上八下納里");
+
+    chewing_delete(ctx);
+}
+
+void test_select_candidate_shift_cursor_rearword()
+{
+    ChewingContext *ctx;
+
+    clean_userphrase();
+
+    ctx = chewing_new();
+    start_testcase(ctx);
+    chewing_set_maxChiSymbolLen(ctx, 16);
+    chewing_set_phraseChoiceRearward(ctx, 1);
+    chewing_set_autoShiftCur(ctx, 1);
+
+    type_keystroke_by_string(ctx, "fu g;418 vu84s84xu3");
+    ok_preedit_buffer(ctx, "七上八下那裡");
+
+    type_keystroke_by_string(ctx, "<L><L><L>");
+
+    type_keystroke_by_string(ctx, "<D>1");
+    ok_preedit_buffer(ctx, "七上八下那裡");
+
+    type_keystroke_by_string(ctx, "<D>2");
+
+    ok_preedit_buffer(ctx, "七上八下納裡");
+
+    type_keystroke_by_string(ctx, "<D>2");
+
+    ok_preedit_buffer(ctx, "七上八下納里");
+
+    chewing_delete(ctx);
+}
+
+void test_select_candidate_sorted()
+{
+    ChewingContext *ctx;
+
+    static const char *CAND1[] = {
+        "妙", "廟", "繆", "玅", "謬", "庙", "庿"
+    };
+
+    static const char *CAND2[] = {
+        "廟", "妙", "繆", "玅", "謬", "庙", "庿"
+    };
+
+    clean_userphrase();
+
+    ctx = chewing_new();
+    start_testcase(ctx);
+
+    chewing_set_candPerPage(ctx, 2);
+    chewing_set_spaceAsSelection(ctx, 1);
+    chewing_set_phraseChoiceRearward(ctx, 1);
+    ok(chewing_config_set_int(ctx, "chewing.sort_candidates_by_frequency", 1) == 0, "set config should return OK");
+    type_keystroke_by_string(ctx, "aul4 ");
+    ok_candidate(ctx, CAND1, ARRAY_SIZE(CAND1));
+    type_keystroke_by_string(ctx, "2<E>");
+    type_keystroke_by_string(ctx, "aul4 ");
+    ok_candidate(ctx, CAND2, ARRAY_SIZE(CAND2));
+
+    chewing_delete(ctx);
+}
+
 void test_select_candidate()
 {
     test_select_candidate_no_rearward();
@@ -488,6 +583,9 @@ void test_select_candidate()
     test_select_candidate_in_middle_reaward();
     test_select_candidate_second_page();
     test_select_candidate_second_page_rewind();
+    test_select_candidate_shift_cursor();
+    test_select_candidate_shift_cursor_rearword();
+    test_select_candidate_sorted();
 }
 
 void test_Esc_not_entering_chewing()
@@ -893,13 +991,13 @@ void test_Tab_at_the_end()
     start_testcase(ctx);
 
     type_keystroke_by_string(ctx, "hk4g4u6vu84");
-    ok_preedit_buffer(ctx, "\xE6\xB8\xAC\xE8\xA9\xA6\xE4\xB8\x80\xE4\xB8\x8B" /* 測試一下 */ );
+    ok_preedit_buffer(ctx, "測試一下");
 
     type_keystroke_by_string(ctx, "<T>");
-    ok_preedit_buffer(ctx, "\xE6\xB8\xAC\xE8\xA9\xA6\xE5\x84\x80\xE4\xB8\x8B" /* 測試儀下 */ );
+    ok_preedit_buffer(ctx, "策試一下");
 
     type_keystroke_by_string(ctx, "<T>");
-    ok_preedit_buffer(ctx, "\xE6\xB8\xAC\xE8\xA9\xA6\xE4\xB8\x80\xE4\xB8\x8B" /* 測試一下 */ );
+    ok_preedit_buffer(ctx, "測試儀下");
 
     chewing_delete(ctx);
 }
@@ -1112,7 +1210,7 @@ void test_ShiftSpace()
     ok_commit_buffer(ctx, " ");
 
     type_keystroke_by_string(ctx, "hk4 <E>");
-    ok_commit_buffer(ctx, "冊 ");
+    ok_commit_buffer(ctx, "測 ");
 
     chewing_set_ChiEngMode(ctx, SYMBOL_MODE);
     type_keystroke_by_string(ctx, "a ");
@@ -1154,7 +1252,7 @@ void test_ShiftSpaceDisabled()
     ok_commit_buffer(ctx, " ");
 
     type_keystroke_by_string(ctx, "hk4 <E>");
-    ok_commit_buffer(ctx, "冊 ");
+    ok_commit_buffer(ctx, "測 ");
 
     chewing_set_ChiEngMode(ctx, SYMBOL_MODE);
     type_keystroke_by_string(ctx, "a ");
@@ -1310,7 +1408,7 @@ void test_Space_selection_symbol()
     ok(strcmp(const_buf, CAND_2) == 0, "second candidate list head `%s' shall be `%s'", const_buf, CAND_2);
 
     /* rollover */
-    type_keystroke_by_string(ctx, " ");
+    type_keystroke_by_string(ctx, "     ");
     chewing_cand_Enumerate(ctx);
     const_buf = chewing_cand_String_static(ctx);
     ok(strcmp(const_buf, CAND_1) == 0, "first candidate list head `%s' shall be `%s'", const_buf, CAND_1);
@@ -1331,7 +1429,7 @@ void test_Space_selection_insert_eng_mode()
     type_keystroke_by_string(ctx, "hk4");
     chewing_set_ChiEngMode(ctx, SYMBOL_MODE);
     type_keystroke_by_string(ctx, " j");
-    ok_preedit_buffer(ctx, "冊 j");
+    ok_preedit_buffer(ctx, "測 j");
 
     chewing_delete(ctx);
 }
@@ -1527,6 +1625,26 @@ void test_bopomofo_buffer()
     chewing_delete(ctx);
 }
 
+void test_five_word_phrase()
+{
+    ChewingContext *ctx;
+    IntervalType it;
+
+    ctx = chewing_new();
+    start_testcase(ctx);
+
+    type_keystroke_by_string(ctx, "g4ru,442u4fu.6b4" /* ㄕˋ ㄐㄧㄝˋ ㄉㄧˋ ㄑㄧㄡˊ ㄖˋ */ );
+    ok_preedit_buffer(ctx, "世界地球日");
+
+    chewing_interval_Enumerate(ctx);
+
+    ok(chewing_interval_hasNext(ctx) == 1, "shall have next interval");
+    chewing_interval_Get(ctx, &it);
+    ok(it.from == 0 && it.to == 5, "interval (%d, %d) shall be (0, 5)", it.from, it.to);
+
+    chewing_delete(ctx);
+}
+
 void test_longest_phrase()
 {
     ChewingContext *ctx;
@@ -1560,7 +1678,7 @@ void test_auto_commit_phrase()
     chewing_set_maxChiSymbolLen(ctx, 3);
 
     type_keystroke_by_string(ctx, "hk4g4<L><T><L><D>1<EN>`31hk4" /* 測試，測 */ );
-    ok_preedit_buffer(ctx, "，冊");
+    ok_preedit_buffer(ctx, "，測");
     ok_commit_buffer(ctx, "測試");
     type_keystroke_by_string(ctx, "g4" /* 試 */ );
     ok_preedit_buffer(ctx, "，測試");
@@ -1683,7 +1801,7 @@ void test_KB_HSU()
     ok_bopomofo_buffer(ctx, "\xE3\x84\x8D" /* ㄍ */ );
     type_keystroke_by_string(ctx, " "); /* convert "ㄍ" to "ㄜ" */
     ok_bopomofo_buffer(ctx, "");
-    ok_preedit_buffer(ctx, "\xE9\x98\xBF" /* 阿 */);
+    ok_preedit_buffer(ctx, "婀");
     chewing_clean_preedit_buf(ctx);
 
     type_keystroke_by_string(ctx, "n");
@@ -1697,14 +1815,14 @@ void test_KB_HSU()
     ok_bopomofo_buffer(ctx, "\xE3\x84\x8E" /* ㄎ */ );
     type_keystroke_by_string(ctx, " "); /* convert "ㄎ" to "ㄤ" */
     ok_bopomofo_buffer(ctx, "");
-    ok_preedit_buffer(ctx, "\xE9\xAA\xAF" /* 骯 */);
+    ok_preedit_buffer(ctx, "腌");
     chewing_clean_preedit_buf(ctx);
 
     type_keystroke_by_string(ctx, "l");
     ok_bopomofo_buffer(ctx, "\xE3\x84\x8C" /* ㄌ */);
     type_keystroke_by_string(ctx, "f"); /* convert "ㄌ" to "ㄦ" */
     ok_bopomofo_buffer(ctx, "");
-    ok_preedit_buffer(ctx, "\xE7\x88\xBE" /* 爾 */);
+    ok_preedit_buffer(ctx, "耳");
     chewing_clean_preedit_buf(ctx);
 
     type_keystroke_by_string(ctx, "g");
@@ -1732,7 +1850,7 @@ void test_KB_HSU()
     type_keystroke_by_string(ctx, "e");
     ok_bopomofo_buffer(ctx, "ㄐㄧㄝ");
     type_keystroke_by_string(ctx, "j");
-    ok_preedit_buffer(ctx, "界");  /* convert "ㄍㄧㄝ" to "ㄐㄧㄝ" */
+    ok_preedit_buffer(ctx, "藉");  /* convert "ㄍㄧㄝ" to "ㄐㄧㄝ" */
     chewing_clean_preedit_buf(ctx);
 
     type_keystroke_by_string(ctx, "g");
@@ -1771,7 +1889,7 @@ void test_KB_HSU_example()
     ok_preedit_buffer(ctx, "本中心訂於明日開張");
     chewing_clean_preedit_buf(ctx);
 
-    type_keystroke_by_string(ctx, "xhfjxl cen <D>2vedxkjnefnldhwfhwfdejuljgxl dxdcx ");
+    type_keystroke_by_string(ctx, "xhfjxl cen <D>2vedxkjnefnldhwfhwfdej<D>1uljgxl dxdcx ");
     ok_preedit_buffer(ctx, "我衷心期望你能好好地用功讀書");
     chewing_clean_preedit_buf(ctx);
 
@@ -1783,7 +1901,7 @@ void test_KB_HSU_example()
     ok_preedit_buffer(ctx, "盡自己的力量");
     chewing_clean_preedit_buf(ctx);
 
-    type_keystroke_by_string(ctx, "jenjzjjefdgsfkdjem ");
+    type_keystroke_by_string(ctx, "jenj<D>1zjjefdgsfkdjem ");
     ok_preedit_buffer(ctx, "進自己的房間");
     chewing_clean_preedit_buf(ctx);
 
@@ -1807,7 +1925,7 @@ void test_KB_HSU_example()
     ok_preedit_buffer(ctx, "三歲到五歲的小孩五到十人");
     chewing_clean_preedit_buf(ctx);
 
-    type_keystroke_by_string(ctx, "jxl cewjdxl lxjxfdxmjleojcde cekj<D><D>1xfnxljcdve hwjjeoflod");
+    type_keystroke_by_string(ctx, "jxl cewjdxl lxjxfdxmjleojcde cekj<D><D>1xfnxljcdve hwj<D>1jeoflod");
     ok_preedit_buffer(ctx, "忠孝東路五段六十一巷五弄十七號九樓");
     chewing_clean_preedit_buf(ctx);
 
@@ -1945,6 +2063,25 @@ void test_KB_HSU_JVC()
     chewing_delete(ctx);
 }
 
+void test_KB_ET()
+{
+    ChewingContext *ctx;
+    ctx = chewing_new();
+    start_testcase(ctx);
+
+    chewing_set_KBType(ctx, KB_ET);
+    chewing_set_maxChiSymbolLen(ctx, 20);
+
+    type_keystroke_by_string(ctx, "hx3b8 p04mz ;y37u4'z3de4<E>");
+    ok_commit_buffer(ctx, "虎斑胖貓走去草地");
+    type_keystroke_by_string(ctx, "k84nez3=2;i4/x4/04tez4uw4j04ta <D>3ge84lew4ce9 ce3<E>");
+    ok_commit_buffer(ctx, "看鳥兒在樹上跳躍讓她見獵心喜");
+    type_keystroke_by_string(ctx, "vx- <D>9sxo ,r1/9 ;1ez4fq b9 .x 7u4<E>");
+    ok_commit_buffer(ctx, "弓縮著身子要飛奔出去");
+
+    chewing_delete(ctx);
+}
+
 void test_KB_ET26()
 {
     ChewingContext *ctx;
@@ -1985,7 +2122,7 @@ void test_KB_ET26()
     ok_bopomofo_buffer(ctx, "\xE3\x84\x8A" /* ㄊ */ );
     type_keystroke_by_string(ctx, " "); /* convert "ㄊ" to "ㄤ" */
     ok_bopomofo_buffer(ctx, "");
-    ok_preedit_buffer(ctx, "\xE9\xAA\xAF" /* 骯 */);
+    ok_preedit_buffer(ctx, "腌");
     chewing_clean_preedit_buf(ctx);
 
     type_keystroke_by_string(ctx, "l");
@@ -1999,7 +2136,7 @@ void test_KB_ET26()
     ok_bopomofo_buffer(ctx, "\xE3\x84\x8F" /* ㄏ */);
     type_keystroke_by_string(ctx, "j"); /* convert "ㄏ" to "ㄦ" */
     ok_bopomofo_buffer(ctx, "");
-    ok_preedit_buffer(ctx, "\xE7\x88\xBE" /* 爾 */);
+    ok_preedit_buffer(ctx, "耳");
     chewing_clean_preedit_buf(ctx);
 
     type_keystroke_by_string(ctx, "g");
@@ -2138,6 +2275,43 @@ void test_KB_DACHEN_CP26()
     chewing_delete(ctx);
 }
 
+void test_KB_GIN_YIEH()
+{
+    ChewingContext *ctx;
+    ctx = chewing_new();
+    start_testcase(ctx);
+
+    chewing_set_KBType(ctx, KB_GIN_YIEH);
+    chewing_set_maxChiSymbolLen(ctx, 20);
+
+    type_keystroke_by_string(ctx, "v[a20 w;zsl u.ag'zjla3-z<E>");
+    ok_commit_buffer(ctx, "虎斑胖貓走去草地");
+    type_keystroke_by_string(ctx, "f0zd-la=qu9zh[zh;ze-lz',zn;ze8 <D>3t-0zc-,zb-p b-a<E>");
+    ok_commit_buffer(ctx, "看鳥兒在樹上跳躍讓她見獵心喜");
+    type_keystroke_by_string(ctx, "r[/ <D>9m[i 6k1hp u1-lzxo 2p y[ g'z<E>");
+    ok_commit_buffer(ctx, "弓縮著身子要飛奔出去");
+
+    chewing_delete(ctx);
+}
+
+void test_KB_IBM()
+{
+    ChewingContext *ctx;
+    ctx = chewing_new();
+    start_testcase(ctx);
+
+    chewing_set_KBType(ctx, KB_IBM);
+    chewing_set_maxChiSymbolLen(ctx, 20);
+
+    type_keystroke_by_string(ctx, "-s,1x 2v.3; iz,wd.o;,5a.<E>");
+    ok_commit_buffer(ctx, "虎斑胖貓走去草地");
+    type_keystroke_by_string(ctx, "0x.7a;,nmik.ys.yv.6a;.dj.uv.6f <D>3qax.8aj.eac ea,<E>");
+    ok_commit_buffer(ctx, "看鳥兒在樹上跳躍讓她見獵心喜");
+    type_keystroke_by_string(ctx, "9sb <D>9psg rh/yc i/a;.4l 1c ts wd.<E>");
+    ok_commit_buffer(ctx, "弓縮著身子要飛奔出去");
+
+    chewing_delete(ctx);
+}
 
 void test_KB_HANYU()
 {
@@ -2340,9 +2514,12 @@ void test_KB()
     test_KB_HSU_choice_append();
     test_KB_HSU_choice_append_select();
     test_KB_HSU_JVC();
+    test_KB_ET();
     test_KB_ET26();
     test_KB_ET26_choice_append();
     test_KB_DACHEN_CP26();
+    test_KB_GIN_YIEH();
+    test_KB_IBM();
     test_KB_DVORAK();
     test_KB_DVORAK_HSU();
     test_KB_COLEMAK();
@@ -2485,6 +2662,7 @@ int main(int argc, char *argv[])
     test_get_phoneSeq();
     test_bopomofo_buffer();
 
+    test_five_word_phrase();
     test_longest_phrase();
     test_auto_commit();
 
