@@ -1,11 +1,11 @@
-use std::ffi::{c_char, CStr};
+use std::ffi::{CStr, c_char};
 
 use chewing::zhuyin::Syllable;
 
 /// # Safety
 ///
 /// This function should be called with valid pointers.
-#[no_mangle]
+#[unsafe(no_mangle)]
 pub unsafe extern "C" fn UintFromPhone(phone: *const c_char) -> u16 {
     let cstr = unsafe { CStr::from_ptr(phone) };
     let rstr = match cstr.to_str() {
