@@ -1,9 +1,5 @@
-use anyhow::{Context, Result};
-use chewing::{
-    dictionary::{DictionaryBuilder, DictionaryInfo, SqliteDictionaryBuilder, TrieBuilder},
-    zhuyin::{Bopomofo, Syllable},
-};
 use std::{
+    any::Any,
     error::Error,
     fmt::Display,
     fs::{self, File},
@@ -11,6 +7,16 @@ use std::{
     path::Path,
 };
 
+#[cfg(not(feature = "sqlite"))]
+use anyhow::bail;
+use anyhow::{Context, Result, anyhow};
+#[cfg(feature = "sqlite")]
+use chewing::dictionary::SqliteDictionaryBuilder;
+use chewing::{
+    dictionary::{DictionaryBuilder, DictionaryInfo, TrieBuilder},
+    zhuyin::{Bopomofo, Syllable},
+};
+
 use crate::flags;
 
 #[derive(Debug)]
@@ -61,40 +67,72 @@ impl<T> IntoParseError<T> for Result<T> {
 
 pub(crate) fn run(args: flags::InitDatabase) -> Result<()> {
     let mut builder: Box<dyn DictionaryBuilder> = match args.db_type {
-        flags::DbType::Sqlite => Box::new(SqliteDictionaryBuilder::new()),
+        flags::DbType::Sqlite => {
+            #[cfg(feature = "sqlite")]
+            {
+                Box::new(SqliteDictionaryBuilder::new())
+            }
+            #[cfg(not(feature = "sqlite"))]
+            bail!("sqlite3 dictionary format support was not enabled.");
+        }
         flags::DbType::Trie => Box::new(TrieBuilder::new()),
     };
 
-    builder.set_info(DictionaryInfo {
-        name: args.name,
-        copyright: args.copyright,
-        license: args.license,
-        version: args.version,
-        software: format!("{} {}", env!("CARGO_PKG_NAME"), env!("CARGO_PKG_VERSION")),
-    })?;
+    let mut name = args.name;
+    let mut copyright = args.copyright;
+    let mut license = args.license;
+    let mut version = args.version;
+    let software = format!("{} {}", env!("CARGO_PKG_NAME"), env!("CARGO_PKG_VERSION"));
 
     let tsi = File::open(args.tsi_src)?;
     let reader = BufReader::new(tsi);
     let delimiter = if args.csv { ',' } else { ' ' };
+    let mut read_front_matter = true;
     let mut errors = vec![];
 
     for (line_num, line) in reader.lines().enumerate() {
-        if args.csv && line_num == 0 {
-            continue;
-        }
         let line = line?;
         let line = line.trim();
-        if line.is_empty() || line.starts_with('#') {
+        if line.is_empty() {
             continue;
         }
-        match parse_line(line_num, delimiter, &line, args.keep_word_freq, args.fix) {
-            Ok((syllables, phrase, freq)) => builder.insert(&syllables, (phrase, freq).into())?,
+        // Read front matter until first non-comment line
+        if read_front_matter && !line.starts_with('#') {
+            read_front_matter = false;
+        } else if read_front_matter {
+            let Some((key, value)) = line.trim_start_matches('#').trim().split_once(delimiter)
+            else {
+                errors.push(parse_error(line_num, "invalid metadata").into());
+                continue;
+            };
+            let value = value.trim_end_matches(delimiter).to_string();
+            match key.trim() {
+                "dc:title" => name = value,
+                "dc:rights" => copyright = value,
+                "dc:license" => license = value,
+                "dc:identifier" => version = value,
+                _ => (),
+            }
+            continue;
+        } else if line.starts_with('#') {
+            continue;
+        }
+        match parse_line(line_num, delimiter, &line, args.fix) {
+            Ok((syllables, phrase, freq)) => {
+                if syllables.len() != phrase.chars().count() {
+                    errors.push(
+                        anyhow!("word count doesn't match").context(parse_error(line_num, line)),
+                    );
+                    continue;
+                }
+                builder.insert(&syllables, (phrase, freq).into())?;
+            }
             Err(error) => errors.push(error),
         };
     }
     if !errors.is_empty() {
         for err in errors {
-            eprintln!("{}", err);
+            eprintln!("{:#}", err);
         }
         if !args.fix {
             eprintln!("Hint: Use --fix to automatically fix common errors");
@@ -107,11 +145,25 @@ pub(crate) fn run(args: flags::InitDatabase) -> Result<()> {
     if path.exists() {
         fs::remove_file(path).context("unable to overwrite output")?;
     }
+
+    let info = DictionaryInfo {
+        name,
+        copyright,
+        license,
+        version,
+        software,
+    };
+    builder.set_info(info.clone())?;
+
     builder.build(path)?;
 
-    if let Some(trie_builder) = builder.as_any().downcast_ref::<TrieBuilder>() {
+    if let Some(trie_builder) = (builder as Box<dyn Any>).downcast_ref::<TrieBuilder>() {
         let stats = trie_builder.statistics();
         eprintln!("== Trie Dictionary Statistics ==");
+        eprintln!("Name                 : {}", info.name);
+        eprintln!("Copyright            : {}", info.copyright);
+        eprintln!("License              : {}", info.license);
+        eprintln!("Version              : {}", info.version);
         eprintln!("Node count           : {}", stats.node_count);
         eprintln!("Leaf count           : {}", stats.leaf_count);
         eprintln!("Phrase count         : {}", stats.phrase_count);
@@ -128,7 +180,6 @@ fn parse_line(
     line_num: usize,
     delimiter: char,
     line: &str,
-    keep_word_freq: bool,
     fix: bool,
 ) -> Result<(Vec<Syllable>, &str, u32)> {
     let phrase = line
@@ -137,18 +188,15 @@ fn parse_line(
         .ok_or(parse_error(line_num, line))?
         .trim_matches('"');
 
-    let freq: u32 = match phrase.chars().count() {
-        1 if !keep_word_freq => 0,
-        _ => line
-            .split(delimiter)
-            .filter(|s| !s.is_empty())
-            .nth(1)
-            .ok_or(parse_error(line_num, line))?
-            .trim_matches('"')
-            .parse()
-            .context("Unable to parse frequency")
-            .parse_error(line_num, line)?,
-    };
+    let freq: u32 = line
+        .split(delimiter)
+        .filter(|s| !s.is_empty())
+        .nth(1)
+        .ok_or(parse_error(line_num, line))?
+        .trim_matches('"')
+        .parse()
+        .context("Unable to parse frequency")
+        .parse_error(line_num, line)?;
 
     let mut syllables = vec![];
 
@@ -205,7 +253,7 @@ mod tests {
     #[test]
     fn parse_ssv() {
         let line = "鑰匙 668 ㄧㄠˋ ㄔˊ # not official";
-        if let Ok((syllables, phrase, freq)) = parse_line(0, ' ', &line, false, false) {
+        if let Ok((syllables, phrase, freq)) = parse_line(0, ' ', &line, false) {
             assert_eq!(syllables, vec![syl![I, AU, TONE4], syl![CH, TONE2]]);
             assert_eq!("鑰匙", phrase);
             assert_eq!(668, freq);
@@ -217,7 +265,7 @@ mod tests {
     #[test]
     fn parse_ssv_multiple_whitespace() {
         let line = "鑰匙     668 ㄧㄠˋ ㄔˊ # not official";
-        if let Ok((syllables, phrase, freq)) = parse_line(0, ' ', &line, false, false) {
+        if let Ok((syllables, phrase, freq)) = parse_line(0, ' ', &line, false) {
             assert_eq!(syllables, vec![syl![I, AU, TONE4], syl![CH, TONE2]]);
             assert_eq!("鑰匙", phrase);
             assert_eq!(668, freq);
@@ -229,7 +277,7 @@ mod tests {
     #[test]
     fn parse_ssv_syllable_errors() {
         let line = "地永天長 50 ㄉ一ˋ ㄩㄥˇ ㄊ一ㄢ ㄔ丫ˊ";
-        if let Ok((syllables, phrase, freq)) = parse_line(0, ' ', &line, false, true) {
+        if let Ok((syllables, phrase, freq)) = parse_line(0, ' ', &line, true) {
             assert_eq!(
                 syllables,
                 vec![
@@ -249,7 +297,7 @@ mod tests {
     #[test]
     fn parse_csv() {
         let line = "鑰匙,668,ㄧㄠˋ ㄔˊ # not official";
-        if let Ok((syllables, phrase, freq)) = parse_line(0, ',', &line, false, false) {
+        if let Ok((syllables, phrase, freq)) = parse_line(0, ',', &line, false) {
             assert_eq!(syllables, vec![syl![I, AU, TONE4], syl![CH, TONE2]]);
             assert_eq!("鑰匙", phrase);
             assert_eq!(668, freq);
@@ -261,7 +309,7 @@ mod tests {
     #[test]
     fn parse_csv_quoted() {
         let line = "\"鑰匙\",668,\"ㄧㄠˋ ㄔˊ # not official\"";
-        if let Ok((syllables, phrase, freq)) = parse_line(0, ',', &line, false, false) {
+        if let Ok((syllables, phrase, freq)) = parse_line(0, ',', &line, false) {
             assert_eq!(syllables, vec![syl![I, AU, TONE4], syl![CH, TONE2]]);
             assert_eq!("鑰匙", phrase);
             assert_eq!(668, freq);
